@@ -54,10 +54,15 @@ psycopg2.extensions.register_type(psycopg2.extensions.UNICODEARRAY)
 
 def _connect(dsn, autocommit=False):
     o = psycopg2.connect(dsn)
-    o.set_client_encoding('UTF8')
-    if autocommit:
-        o.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
-    return o
+    try:
+        o.set_client_encoding('UTF8')
+        if autocommit:
+            o.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
+    except:
+        o.close()
+        raise
+    else:
+        return o
 
 
 def _compose_select(kwargs, table):
@@ -452,9 +457,10 @@ class Database(Connection):
                 setattr(cls, attr_name, {})
             instances = getattr(cls, attr_name)
             if (dsn, dictrows) not in instances:
-                instance = instances[(dsn, dictrows)] = object.__new__(cls)
+                instance = object.__new__(cls)
                 Connection.__init__(instance, dsn, dictrows)
                 instance._Database__is_frozen = True
+                instances[(dsn, dictrows)] = instance
         return instances[(dsn, dictrows)]
 
     def __init__(self, dsn, dictrows=False):
@@ -469,7 +475,7 @@ class Database(Connection):
         object.__setattr__(self, key, value)
 
     def close(self):
-        raise TypeError("'Database' instances can not be closed")
+        pass
 
 
 class Cursor:
@@ -477,26 +483,28 @@ class Cursor:
 
     def __init__(self, dsn, query, query_params=[], buffer_size=10000, 
                  dictrows=False):
-        self.__connection = _connect(dsn)
-        self.__cursor = None
-        self.__buffer = iter([])
-        self.__owning_thread = threading.current_thread()
-        self.__query = query
-        self.__query_params = query_params
-        self.__buffer_size = buffer_size
-        self.__dictrows = dictrows
+        self._dsn = dsn
+        self._query = query
+        self._query_params = query_params
+        self._buffer_size = buffer_size
+        self._dictrows = dictrows
+        self._connection = _connect(dsn)
+        self._cursor = None
+        self._buffer = iter([])
+        if __debug__:
+            self._owning_thread = threading.current_thread()
 
     def __iter__(self):
         return self
 
     def __next__(self):
-        assert self.__owning_thread is threading.current_thread(), \
+        assert self._owning_thread is threading.current_thread(), \
             "'Cursor' instances can not be shared between threads"
         try:
-            return next(self.__buffer)
+            return next(self._buffer)
         except StopIteration:
-            self.__fetch()
-            return next(self.__buffer)
+            self._fetch()
+            return next(self._buffer)
 
     def __enter__(self):
         return self
@@ -504,25 +512,27 @@ class Cursor:
     def __exit__(self, *exc_info):
         self.close()
 
-    def __fetch(self):
-        if self.__cursor is None:
-            self.__cursor = self.__connection.cursor(
+    def _fetch(self):
+        if self._cursor is None:
+            self._cursor = self._connection.cursor(
                 'curiousorm_cursor', 
                 cursor_factory=(
-                    DictCursor if self.__dictrows else NamedTupleCursor))
-            self.__cursor.arraysize = self.__buffer_size
-            self.__cursor.execute(self.__query, self.__query_params)
-        self.__buffer = iter(self.__cursor.fetchmany(self.__buffer_size))
+                    DictCursor if self._dictrows else NamedTupleCursor))
+            self._cursor.arraysize = self._buffer_size
+            self._cursor.execute(self._query, self._query_params)
+        self._buffer = iter(self._cursor.fetchmany(self._buffer_size))
 
     next = __next__
 
     def close(self):
-        assert self.__owning_thread is threading.current_thread(), \
+        assert self._owning_thread is threading.current_thread(), \
             "'Cursor' instances can not be shared between threads"
-        self.__buffer = None
-        if self.__cursor:
-            self.__cursor.close()
-        self.__connection.close()
+        self._buffer = None
+        try:
+            self._cursor.close()
+        except:
+            pass
+        self._connection.close()
 
 
 def retry_on_deadlock(action):
